--
-- InteractiveControl v4.1
--
-- Interface: 1.4.4.0 1.4.4RC8
--
-- Specialization for an interactive control
--
-- SFM-Modding
-- @author:      Manuel Leithner
-- @date:        15/05/2013
-- @version:     v2.0
-- @history:     v1.0 - initial implementation
--               v2.0 - convert to LS2011 and some bugfixes
--               v3.0 - convert to LS2013 and bugfixes
-- 20/02/2017	 v4.0 - convert to LS2017 and bugfixes
-- 30/07/2017	 v4.1 - changes as see below
--
-- v4.0 Converted and adapted to LS/FS 2017 including MP Support.  -  GtX | Andy
-- v4.1 Added option to change IC Mode (Quick Access or Normal). Shorten script and remove unused items from LS11 & LS13.  -  GtX | Andy
--
-- Support for v4.0 and above.  @  http://ls-modcompany.com
--
-- @NOTES for 2017 Update: Thankyou to Philipp and Ralp @ LS-Modcompany for MP Testing and also Toranion @ PC-SG for his assistance in MP Testing.
--
-- free for noncommerical-usage
--
--[[

*********************************************************************************************************************************************************************************
*************																																									*
* IMPORTANT *	If this script is used with the "numberPlates.lua" you will need to have the INTERACTIVE CONTROL (IC) in the ON position to be able to register the plate.		* 
*************																																									*
*********************************************************************************************************************************************************************************

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
Minimum Requirements:		Version 4.1 and above.                                                                                                                                           
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

InteractiveComponentInterface.lua		-		This "MUST" be in scripts folder.
InteractiveControl.lua
InteractiveWindows.lua
InteractiveButtons.lua

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
Optional:					Version 4.1 and above.                                                                                                                                           
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

InteractiveSounds.lua
InteractiveExternal.lua

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
Example for modDesc:		Version 4.1 and above.                                                                                                                                           
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

<specializations>                                                                                                                                                                         
	<specialization name="interactiveControl" 		className="InteractiveControl"  	filename="scripts/InteractiveControl.lua" />		-		Minimum Requirement                   
	<specialization name="interactiveWindows" 		className="InteractiveWindows"  	filename="scripts/InteractiveWindows.lua" />		-		Minimum Requirement                   
	<specialization name="interactiveButtons" 		className="InteractiveButtons"  	filename="scripts/InteractiveButtons.lua" />		-		Minimum Requirement                   
	<specialization name="interactiveSounds" 		className="InteractiveSounds"  		filename="scripts/InteractiveSounds.lua" />			-		Optional                              
	<specialization name="interactiveExternal"		className="InteractiveExternal"		filename="scripts/InteractiveExternal.lua" />		-		Optional                              
</specializations>                                                                                                                                                                        


<vehicleTypes>                                                                                                                                                                            
	<type name="myGreatMod" className="Vehicle" filename="$dataS/scripts/vehicles/Vehicle.lua">                                                                                           
		<specialization name="attacherJoints" />		----------                                                                                                                        
		<specialization name="lights" />						--                                                                                                                        
		<specialization name="animatedVehicle" />				--                                                                                                                        
		<specialization name="cylindered" />					--                                                                                                                        
		<specialization name="motorized" />						--                                                                                                                        
		<specialization name="steerable" />	                    --                                                                                                                        
		<specialization name="drivable" />						--       Minimum Requirements                                                                                             
		<specialization name="honk" />							--                                                                                                                        
		<specialization name="washable" />						--                                                                                                                        
		<specialization name="mountable" />						--                                                                                                                        
		<specialization name="interactiveControl" />			--                                                                                                                        
		<specialization name="interactiveWindows" />			--                                                                                                                        
		<specialization name="interactiveButtons" />	----------                                                                                                                        

		
		<specialization name="InteractiveSounds" />		----------			Optional                                                                                                      
		<specialization name="interactiveExternal" />	----------			Optional		                                                                                              
	</type>                                                                                                                                                                               
</vehicleTypes>                                                                                                                                                                           

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
Support and questions for this version of the Script available: http://ls-modcompany.com	GtX | Andy
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

--]]

originalInputBindingUpdate = InputBinding.update;

InputBinding.update = function(dt)
    InputBinding.accumMouseMovementXBackUp = InputBinding.accumMouseMovementX;
    InputBinding.accumMouseMovementYBackUp = InputBinding.accumMouseMovementY;
    originalInputBindingUpdate(dt);
end

InteractiveControl = {};

function InteractiveControl.prerequisitesPresent(specializations)
	if not SpecializationUtil.hasSpecialization(Steerable, specializations) then print("Warning: Specialization InteractiveControl v4.1 needs the specialization Steerable."); end;
    return SpecializationUtil.hasSpecialization(Steerable, specializations);
end;

function InteractiveControl:load(xmlFile)

    source(Utils.getFilename("scripts/InteractiveComponentInterface.lua", self.baseDirectory));

    self.doActionOnObject = SpecializationUtil.callSpecializationsFunction("doActionOnObject");

    self.interactiveObjects = {};

    self.indoorCamIndex = 2;
    self.outdoorCamIndex = 1;

    self.lastMouseXPos = 0;
    self.lastMouseYPos = 0;

    self.foundInteractiveObject = nil;
    self.isMouseActive = false;
	
	self.accessMode = false
	
	self.icModeMessage = g_i18n:getText("InteractiveControl_ModeSelect");
	self.icQuick = g_i18n:getText("InteractiveControl_Quick");
    self.icNormal = g_i18n:getText("button_normal");  -- Giants l10n (So not needed in modDesc)
	
end;


function InteractiveControl:postLoad(savegame)
    if savegame ~= nil then
		local i=0;
		while true do
			local objKey = savegame.key .. string.format(".interactiveObject(%d)", i);
			if not hasXMLProperty(savegame.xmlFile, objKey) then
				break;
			end;
			local id = getXMLInt(savegame.xmlFile, objKey.."#id");
			if id ~= nil then
				local state = Utils.getNoNil(getXMLBool(savegame.xmlFile, objKey.."#state"), false);
				local iObj = self.interactiveObjects[id];
				if iObj ~= nil then
					iObj:doAction(true, state);
				end;
			end;
			i = i +1;
		end;
    end
end

function InteractiveControl:delete()
    for _,v in pairs(self.interactiveObjects) do
        v:delete();
    end;
end;

function InteractiveControl:readStream(streamId, connection)
    local icCount = streamReadInt8(streamId);
    for i=1, icCount do
        local isOpen = streamReadBool(streamId);
        if self.interactiveObjects[i].synch then
            self.interactiveObjects[i]:doAction(true, isOpen);
        end;
    end;
end;

function InteractiveControl:writeStream(streamId, connection)
    streamWriteInt8(streamId, table.getn(self.interactiveObjects));
    for k,v in pairs(self.interactiveObjects) do
        streamWriteBool(streamId, v.isOpen);
    end;
end;

function InteractiveControl:mouseEvent(posX, posY, isDown, isUp, button)

    self.lastMouseXPos = posX;
    self.lastMouseYPos = posY;

    if isDown then
        if Input.isMouseButtonPressed(Input.MOUSE_BUTTON_LEFT) and self.foundInteractiveObject ~= nil then
            self:doActionOnObject(self.foundInteractiveObject);
        end;
        local currentCam = self.cameras[self.camIndex];
        if currentCam.allowTranslation then
            if Input.isMouseButtonPressed(Input.MOUSE_BUTTON_WHEEL_UP) then
                currentCam:zoomSmoothly(-0.75);
            elseif Input.isMouseButtonPressed(Input.MOUSE_BUTTON_WHEEL_DOWN) then
                currentCam:zoomSmoothly(0.75);
            end;
        end;
    end;

    for _,v in pairs(self.interactiveObjects) do
        v:mouseEvent(posX, posY, isDown, isUp, button);
    end;    
end;

function InteractiveControl:keyEvent(unicode, sym, modifier, isDown)
    for _,v in pairs(self.interactiveObjects) do
        v:keyEvent(unicode, sym, modifier, isDown);
    end;
end;

function InteractiveControl:update(dt)
    if self:getIsActive() then

        self.foundInteractiveObject = nil;
        local icObject = nil;
        for k,v in pairs(self.interactiveObjects) do
            v:update(dt);
        end;
	
		if self.isClient and self:getIsActiveForInput(false) and not self:hasInputConflictWithSelection() then

			if InputBinding.hasEvent(InputBinding.INTERACTIVE_CONTROL_MODE) then
				if self.accessMode == false then
					self.accessMode = true
				else
					self.accessMode = false
				end;
			end;

			if self.accessMode then
				if InputBinding.hasEvent(InputBinding.INTERACTIVE_CONTROL_SWITCH) then
					self.isMouseActive = not self.isMouseActive;					
				end
			elseif not self.accessMode then
				if InputBinding.isPressed(InputBinding.INTERACTIVE_CONTROL_SWITCH) then
					self.isMouseActive = true;
				else
					self.isMouseActive = false;
				end
			end
			
            if not self.isMouseActive then
                InputBinding.setShowMouseCursor(false);
                self.cameras[self.camIndex].isActivated = true;
                for _,v in pairs(self.interactiveObjects) do
                    v:onExit(dt);
                end;
            end;

            for _,v in pairs(self.interactiveObjects) do
                v:setVisible(self.isMouseActive);
            end;
        end;
		
        if self.isMouseActive then
            local currentCam = self.cameras[self.camIndex];
            if Input.isMouseButtonPressed(Input.MOUSE_BUTTON_RIGHT) then
                InputBinding.wrapMousePositionEnabled = true;
                currentCam.rotX = currentCam.rotX + InputBinding.accumMouseMovementYBackUp; --mouseMovementY;
                currentCam.rotY = currentCam.rotY - InputBinding.accumMouseMovementXBackUp; --mouseMovementX;

                self.cameras[self.indoorCamIndex].isActivated = self.camIndex == self.indoorCamIndex;
                self.cameras[self.outdoorCamIndex].isActivated = self.camIndex == self.outdoorCamIndex;
                InputBinding.setShowMouseCursor(false);
            else
                self.cameras[self.indoorCamIndex].isActivated = self.camIndex ~= self.indoorCamIndex;
                self.cameras[self.outdoorCamIndex].isActivated = self.camIndex ~= self.outdoorCamIndex;
                InputBinding.setShowMouseCursor(true);
            end;
        else
            self.foundInteractiveObject = nil;
        end;
    else
        self.foundInteractiveObject = nil;
    end;

    InputBinding.accumMouseMovementXBackUp = 0;
    InputBinding.accumMouseMovementYBackUp = 0;

end;

function InteractiveControl:updateTick(dt)
end;

function InteractiveControl:doActionOnObject(id, noEventSend)
    if self.interactiveObjects[id].isLocalOnly == nil or not self.interactiveObjects[id].isLocalOnly then
        InteractiveControlEvent.sendEvent(self, id, noEventSend);
    end;
    self.interactiveObjects[id]:doAction(noEventSend);
end;

function InteractiveControl:draw()
    if self:getIsActive() then
        self.foundInteractiveObject = nil;
        local icObject = nil;
        for k,v in pairs(self.interactiveObjects) do
            if self.isMouseActive then
                v:onExit();
                if icObject == nil and self.camIndex == self.indoorCamIndex then
                    local worldX,worldY,worldZ = getWorldTranslation(v.mark);
                    local x,y,z = project(worldX,worldY,worldZ);
                    if z <= 1 then
                        if self.lastMouseXPos > (x-v.size/2) and self.lastMouseXPos < (x+v.size/2) then
                            if self.lastMouseYPos > (y-v.size/2) and self.lastMouseYPos < (y+v.size/2) then
                                local isOverlapped = false;
                                if not isOverlapped then
                                    icObject = v;
                                    self.foundInteractiveObject = k;
                                    break;
                                end;
                            end;
                        end;
                    end;
                end;
            end;
        end;

        if icObject ~= nil then
            icObject:onEnter();
        end;
    end;

    for _,v in pairs(self.interactiveObjects) do
        v:draw();
    end;	
	
	if self.accessMode then
		if self.isMouseActive then
			g_currentMission:addHelpButtonText(g_i18n:getText("InteractiveControl_Off"), InputBinding.INTERACTIVE_CONTROL_SWITCH);
			g_currentMission:addHelpButtonText(string.format(self.icModeMessage, self.icNormal), InputBinding.INTERACTIVE_CONTROL_MODE, nil, GS_PRIO_HIGH)
		else
			g_currentMission:addHelpButtonText(g_i18n:getText("InteractiveControl_On"), InputBinding.INTERACTIVE_CONTROL_SWITCH);
		end;
		
	elseif not self.accessMode then
		if self.isMouseActive then
			g_currentMission:addHelpButtonText(g_i18n:getText("InteractiveControl_Mode_Active"), InputBinding.INTERACTIVE_CONTROL_SWITCH);
			g_currentMission:addHelpButtonText(string.format(self.icModeMessage, self.icQuick), InputBinding.INTERACTIVE_CONTROL_MODE, nil, GS_PRIO_HIGH)
		else
			g_currentMission:addHelpButtonText(g_i18n:getText("InteractiveControl_On"), InputBinding.INTERACTIVE_CONTROL_SWITCH);
		end;
	end;

end;

function InteractiveControl:onLeave()
    self.cameras[self.indoorCamIndex].isActivated = true;
    if g_gui.currentGui == nil then
        InputBinding.setShowMouseCursor(false);
    end;
end;

function InteractiveControl:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)

    local i=0;
    while true do
        local objKey = key..string.format(".interactiveObject(%d)", i);
        if not hasXMLProperty(xmlFile, objKey) then
            break;
        end;
        local id = getXMLInt(xmlFile, objKey.."#id");
        if id ~= nil then
            local state = Utils.getNoNil(getXMLBool(xmlFile, objKey.."#state"), false);
            local iObj = self.interactiveObjects[id];
            if iObj ~= nil then
                iObj:doAction(true, state);
            end;
        end;
        i = i +1;
    end;

    return BaseMission.VEHICLE_LOAD_OK;
end

function InteractiveControl:getSaveAttributesAndNodes(nodeIdent)
    local attributes = "";
    local nodes = "";

    for id, iObj in pairs(self.interactiveObjects) do
        if id > 1 then
            nodes = nodes.."\n";
        end;
        nodes = nodes..nodeIdent..'<interactiveObject id="'..id..'" state="'..tostring(iObj.isOpen)..'"/>';
    end;

    return attributes, nodes;
end

-------------------------------------------------------------------
----------------------- FS/LS 2017 EVENT --------------------------
-------------------------------------------------------------------

InteractiveControlEvent = {};
InteractiveControlEvent_mt = Class(InteractiveControlEvent, Event);

InitEventClass(InteractiveControlEvent, "InteractiveControlEvent");

function InteractiveControlEvent:emptyNew()
    local self = Event:new(InteractiveControlEvent_mt);
    return self;
end;

function InteractiveControlEvent:new(vehicle, interactiveControlID)
    local self = InteractiveControlEvent:emptyNew()
    self.vehicle = vehicle;
    self.interactiveControlID = interactiveControlID;
    return self;
end;

function InteractiveControlEvent:readStream(streamId, connection)
    self.interactiveControlID = streamReadInt8(streamId);
    self.vehicle = readNetworkNodeObject(streamId);
    self:run(connection);
end;

function InteractiveControlEvent:writeStream(streamId, connection)
	writeNetworkNodeObject(streamId, self.vehicle);
    streamWriteInt8(streamId, self.interactiveControlID);
end;

function InteractiveControlEvent:run(connection)
	if self.vehicle ~= nil then
        self.vehicle:doActionOnObject(self.interactiveControlID, true);
    end;
    if not connection:getIsServer() then
        g_server:broadcastEvent(InteractiveControlEvent:new(self.vehicle, self.interactiveControlID), nil, connection, self.vehicle);
    end;
	
end;

function InteractiveControlEvent.sendEvent(vehicle, icObject, noEventSend)
    if noEventSend == nil or noEventSend == false then
        if g_server ~= nil then
            g_server:broadcastEvent(InteractiveControlEvent:new(vehicle, icObject), nil, nil, vehicle);
        else
            g_client:getServerConnection():sendEvent(InteractiveControlEvent:new(vehicle, icObject));
        end;
    end;
end;